@@ -44,7 +44,7 @@ import multiprocessing.connection
 import concurrent.futures
 from contextlib import contextmanager
 
-VERSION = "0.0.9"
+VERSION = "0.1.0"
 
 class StarryError(Exception):
     def __init__(self, output):
@@ -163,15 +163,19 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, database, address, cache=64, log=None):
+    def __init__(self, database, address, cache_size=0, log_file=None, rw_priority=5):
         self.database = os.path.abspath(database)
         self.address = address
+        self.cache_size = cache_size
         self.cache = None
         self.thread_cond = threading.Condition()
         self.thread_lock = threading.RLock()
-        self.write_first = True
-        self.queue = []
-        self.logger = StarryLog(log) if log is not None else None
+        self.write_wait = 0
+        self.read_wait = 0
+        self.rw_priority = rw_priority
+        self.write_queue = []
+        self.read_queue = []
+        self.logger = StarryLog(log_file) if log_file is not None else None
 
     def __del__(self):
         if self.cache is not None:
@@ -184,17 +188,17 @@ class StarryServer(object):
         if not os.path.isdir(self.database):
             os.mkdir(self.database)
 
-        if type(address) is str:
-            _removeFile(address)
+        if type(self.address) is str:
+            _removeFile(self.address)
 
-        if int(cache) > 0:
+        if int(self.cache_size) > 0:
             try:
                 subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
             try:
-                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(cache)} -s {os.path.join(self.database, 'cache.sock')}")
+                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(self.cache_size)} -s {os.path.join(self.database, 'cache.sock')}")
 
                 time.sleep(1)
 
@@ -605,7 +609,8 @@ class StarryServer(object):
         except:
             pass
 
-        self.queue.clear()
+        self.write_queue.clear()
+        self.read_queue.clear()
 
         return (True, "")
 
@@ -965,40 +970,61 @@ class StarryServer(object):
 
         return self._backup(key, check_flag=False)
 
-
-
     @contextmanager
     def _lock(self, mark):
         def _predicate():
-            if len(self.queue) > 0:
-                if mark["mode"] == "r":
-                    if any([q["mode"] == "w" for q in self.queue]):
-                        if self.write_first is True:
-                            return False
+            if mark["mode"] == "r":
+                if self.write_wait > 0 and (self.read_wait / self.write_wait) <= self.rw_priority:
+                    return False
 
-                        if None in mark["members"]:
-                            return False
-
-                        if any([q["mode"] == "w" and (None in q["members"] or len(set(q["members"]) & set(mark["members"])) > 0) for q in self.queue]):
-                            return False
-                else:
+                if len(self.write_queue) > 0:
                     if None in mark["members"]:
                         return False
 
-                    if any([None in q["members"] or len(set(q["members"]) & set(mark["members"])) > 0 for q in self.queue]):
+                    if any([None in write_members or len(set(write_members) & set(mark["members"])) > 0 for write_members in self.write_queue]):
+                        return False
+            else:
+                if self.write_wait > 0 and (self.read_wait / self.write_wait) > self.rw_priority:
+                    return False
+
+                if len(self.read_queue) > 0:
+                    if None in mark["members"]:
+                        return False
+
+                    if any([None in read_members or len(set(read_members) & set(mark["members"])) > 0 for read_members in self.read_queue]):
+                        return False
+                elif len(self.write_queue) > 0:
+                    if None in mark["members"]:
+                        return False
+
+                    if any([None in write_members or len(set(write_members) & set(mark["members"])) > 0 for write_members in self.write_queue]):
                         return False
 
-            self.queue.append(mark)
             return True
 
         with self.thread_cond:
+            if mark["mode"] == "r":
+                self.read_wait += 1
+            else:
+                self.write_wait += 1
+
             self.thread_cond.wait_for(_predicate)
 
+            if mark["mode"] == "r":
+                self.read_queue.append(mark["members"])
+                self.read_wait -= 1
+            else:
+                self.write_queue.append(mark["members"])
+                self.write_wait -= 1
+
         yield
 
         with self.thread_cond:
             try:
-                self.queue.remove(mark)
+                if mark["mode"] == "r":
+                    self.read_queue.remove(mark["members"])
+                else:
+                    self.write_queue.remove(mark["members"])
             except:
                 pass
 
